==> org-lsst-ccs-subsystem-refrig-main-2.5-sources.jar: org/lsst/ccs/subsystem/refrig/SimDevice.java <==
package org.lsst.ccs.subsystem.refrig;

/**
 ******************************************************************************
 **
 **  Simulates the refrigeration hardware.
 **
 **  @author Owen Saxton
 **
 ******************************************************************************
 */
public class SimDevice extends Device {

   /**
    ***************************************************************************
    **
    **  Constants.
    **
    ***************************************************************************
    */
    final static int
        MAIN_POWER_LINE = 0,
        LOAD_POWER_LINE = 1,
        N_DEV_CHANS = 24,
        N_DEV_LINES = 2;

   /**
    ***************************************************************************
    **
    **  Data fields.
    **
    ***************************************************************************
    */
    final static double LOAD_POWER_FCTR = 0.0006667;
    ChannelSim[] chanData = new ChannelSim[N_DEV_CHANS];
    ChannelSim loadPwrChan, loadTmpChan;
    double loadFract = 0.6;
    boolean mainPowerOn, loadPowerOn;
    int lineMask; // Mask of output line numbers on this device


   /**
    ***************************************************************************
    **
    **  Inner container class for channel simulation data.
    **
    ***************************************************************************
    */
    class ChannelSim {

       /**
        ***********************************************************************
        **
        **  Data fields
        **
        ***********************************************************************
        */
        int      hwChan;      // HW channel number
        double   onValue;     // On (operating) value
        double   onTime;      // Time (secs) to reach on value
        double   offValue;    // Off (ambient) value
        double   offTime;     // Time (secs) to reach off value

        double   endValue;    // Ending value
        double   rate;        // Rate of change
        boolean  negative;    // True if nominal change is negative
        double   value;       // Current value
        long     currTime;    // Current system time


       /**
        ***********************************************************************
        **
        **  Constructor
        **
        ***********************************************************************
        */
        public ChannelSim(int hwChan, double onValue, double onTime,
                          double offValue, double offTime)
        {
            this.hwChan   = hwChan;
            this.onValue  = onValue;
            this.onTime   = onTime;
            this.offValue = offValue;
            this.offTime  = offTime;
            currTime      = System.currentTimeMillis();
        }


       /**
        ***********************************************************************
        **
        **  Initializes the channel.
        **
        ***********************************************************************
        */
        void initialize()
        {
            if (this == loadPwrChan) {
                onTime = 0;
                offValue = 0;
                offTime = 0;
            }
            endValue = offValue;
            value = offValue;
        }


       /**
        ***********************************************************************
        **
        **  Reads the channel.
        **
        ***********************************************************************
        */
        void read()
        {
            long thisTime = System.currentTimeMillis();
            double intvl = (thisTime - currTime) / 1000.0;
            currTime = thisTime;
            value += rate * intvl;
            if (negative) {
                if (value < endValue) {
                    value = endValue;
                }
            }
            else {
                if (value > endValue) {
                    value = endValue;
                }
            }
        }


       /**
        ***********************************************************************
        **
        **  Sets channel to power-on value.
        **
        ***********************************************************************
        */
        void powerOn()
        {
            currTime = System.currentTimeMillis();
            rate = 0;
            if (mainPowerOn) {
                endValue = onValue;
                if (onTime != 0) {
                    rate = (onValue - offValue) / onTime;
                }
            }
            else {
                endValue = offValue;
                if (offTime != 0) {
                    rate = (offValue - onValue) / offTime;
                }
            }
            if (rate == 0) {
                value = endValue;
            }
            negative = rate < 0;
        }

    }


   /**
    ***************************************************************************
    **
    **  Constructor.
    **
    ***************************************************************************
    */
    public SimDevice(String devName, String serialNo)
    {
    }


   /**
    ***************************************************************************
    **
    **  Checks a channel's parameters for validity.
    **
    ***************************************************************************
    */
    @Override
    int[] checkChannel(String name, int hwChan, String type, String subtype)
        throws Exception
    {
        if (hwChan < 0 || hwChan >= N_DEV_CHANS) {
            refg.reportError(name, "hw channel number", hwChan);
        }
        String flds[] = subtype.split(":");
        try {
            double onValue = Double.valueOf(flds[0]);
            double onTime = Double.valueOf(flds[1]);
            double offValue = Double.valueOf(flds[2]);
            double offTime = Double.valueOf(flds[3]);
            ChannelSim chan = new ChannelSim(hwChan, onValue, onTime,
                                             offValue, offTime);
            chanData[hwChan] = chan;
            if (name.equals("LoadPower")) {
                loadPwrChan = chan;
            }
            else if (name.equals("LoadTmp")) {
                loadTmpChan = chan;
            }
        }
        catch (Exception e) {
            refg.reportError(name, "subtype", subtype);
        }
                
        return new int[]{0, 0};
    }


   /**
    ***************************************************************************
    **
    **  Performs basic initialization.
    **
    ***************************************************************************
    */
    @Override
    void initialize()
    {
        if (loadPwrChan == null) {
            log.warn("No load power channel defined");
        }
        if (loadTmpChan == null) {
            log.warn("No load temperature channel defined");
        }
        setOnline(true);
        initSensors();
    }


   /**
    ***************************************************************************
    **
    **  Initializes a channel.
    **
    ***************************************************************************
    */
    @Override
    void initChannel(int hwChan, int type, int subtype)
    {
        chanData[hwChan].initialize();
    }


   /**
    ***************************************************************************
    **
    **  Reads a channel.
    **
    ***************************************************************************
    */
    @Override
    double readChannel(int hwChan)
    {
        ChannelSim chan = chanData[hwChan];
        chan.read();

        return chan.value;
    }


   /**
    ***************************************************************************
    **
    **  Checks an output line number.
    **
    ***************************************************************************
    */
    @Override
    void checkLine(String name, int line) throws Exception
    {
        if (line < 0 || line >= N_DEV_LINES) {
            refg.reportError(name, "line number", line);
        }
    }


   /**
    ***************************************************************************
    **
    **  Adds an output line.
    **
    ***************************************************************************
    */
    @Override
    void addLine(int line)
    {
        lineMask |= 1 << line;
    }


   /**
    ***************************************************************************
    **
    **  Sets an output line on or off.
    **
    ***************************************************************************
    */
    @Override
    void setLine(int line, boolean on)
    {
        if (line == MAIN_POWER_LINE) {
            setMainPower(on);
        }
        else if (line == LOAD_POWER_LINE) {
            setLoadPower(on);
        }
    }


   /**
    ***************************************************************************
    **
    **  Gets the set state of an output line.
    **
    ***************************************************************************
    */
    @Override
    boolean isLineSet(int line)
    {
        if (line == MAIN_POWER_LINE) return mainPowerOn;
        else if (line == LOAD_POWER_LINE) return loadPowerOn;
        else return false;
    }


   /**
    ***************************************************************************
    **
    **  Closes the connection.
    **
    ***************************************************************************
    */
    @Override
    void close()
    {
    }


   /**
    ***************************************************************************
    **
    **  Sets the load power fraction
    **
    ***************************************************************************
    */
    public void setLoadFract(double fract)
    {
        loadFract = fract;
        setLoadPower();
    }


   /**
    ***************************************************************************
    **
    **  Turns the main power on or off.
    **
    ***************************************************************************
    */
    void setMainPower(boolean on)
    {
        mainPowerOn = on;
        for (ChannelSim chan : chanData) {
            if (chan == null || chan == loadPwrChan) continue;
            chan.powerOn();
        }
        setLoadPower();
    }
            

   /**
    ***************************************************************************
    **
    **  Turns the load power on or off, setting the correct level.
    **
    ***************************************************************************
    */
    void setLoadPower(boolean on)
    {
        loadPowerOn = on;
        setLoadPower();
    }

    void setLoadPower()
    {
        if (loadPwrChan == null) return;
        double endValue = 0;
        if (mainPowerOn) {
            if (loadPowerOn) {
                endValue = loadFract * loadPwrChan.onValue;
            }
            ChannelSim tc = loadTmpChan;
            if (tc != null && tc.onTime != 0) {
                tc.rate = (tc.onValue - tc.offValue) / tc.onTime
                            + LOAD_POWER_FCTR * endValue;
            }
        }
        loadPwrChan.endValue = endValue;
        loadPwrChan.value = endValue;
    }

}

==> org-lsst-ccs-subsystem-refrig-main-3.0-sources.jar: org/lsst/ccs/subsystem/refrig/SimDevice.java <==
package org.lsst.ccs.subsystem.refrig;

import org.lsst.ccs.subsystem.monitor.Device;

/**
 ******************************************************************************
 **
 **  Simulates the refrigeration hardware.
 **
 **  @author Owen Saxton
 **
 ******************************************************************************
 */
public class SimDevice extends Device {

   /**
    ***************************************************************************
    **
    **  Constants.
    **
    ***************************************************************************
    */
    final static int
        MAIN_POWER_LINE = 0,
        LOAD_POWER_LINE = 1,
        N_DEV_CHANS = 64,
        N_DEV_LINES = 8;

   /**
    ***************************************************************************
    **
    **  Data fields.
    **
    ***************************************************************************
    */
    final static double LOAD_POWER_FCTR = 0.0006667;
    static SimDevice sim;
    static Power power;
    ChannelSim[] chanData = new ChannelSim[N_DEV_CHANS];
    ChannelSim loadTmpChan, loadPwrChan;
    double loadFract = 0.6;
    boolean[] lineOn = new boolean[N_DEV_LINES];


   /**
    ***************************************************************************
    **
    **  Inner class to simulate power supply device with attached load.
    **
    ***************************************************************************
    */
    static class Power extends Device implements PowerDevice {

        private final static int
            N_DEV_CHANS = 3,
            CHAN_WATTS = 0,
            CHAN_VOLTS = 1,
            CHAN_AMPS  = 2;
        private final static double
            LOAD_OHMS = 50;

        double setVolts, setAmps, loadVolts, loadAmps, loadWatts;
        boolean outputOn;

       /**
        ***********************************************************************
        **
        **  Constructor.
        **
        ***********************************************************************
        */
        public Power()
        {
            fullName = "Simulated power supply";
            power = this;
        }


       /**
        ***********************************************************************
        **
        **  Performs basic initialization.
        **
        ***********************************************************************
        */
        @Override
        public void initialize()
        {
            if (!sim.lineOn[LOAD_POWER_LINE]) return;
            setOnline(true);
            initSensors();
        }


       /**
        ***********************************************************************
        **
        **  Closes the connection.
        **
        ***********************************************************************
        */
        @Override
        public void close()
        {
        }


       /**
        ***********************************************************************
        **
        **  Checks a channel's parameters for validity.
        **
        ***********************************************************************
        */
        @Override
        public int[] checkChannel(String name, int hwChan, String type,
                                  String subtype) throws Exception
        {
            if (hwChan < 0 || hwChan >= N_DEV_CHANS) {
                mon.reportError(name, "hw channel number", hwChan);
            }
                    
            return new int[]{0, 0};
        }


       /**
        ***********************************************************************
        **
        **  Reads a channel.
        **
        ***********************************************************************
        */
        @Override
        public double readChannel(int hwChan, int type)
        {
            switch (hwChan) {
            case CHAN_WATTS:
                return power.loadWatts;
            case CHAN_VOLTS:
                return power.loadVolts;
            case CHAN_AMPS:
                return power.loadAmps;
            default:
                return 0;
            }
        }


       /**
        ***********************************************************************
        **
        **  Sets the power supply voltage.
        **
        ***********************************************************************
        */
        @Override
        public void setVoltage(int chan, double value)
        {
            setVolts = value;
            setPower();
        }


       /**
        ***********************************************************************
        **
        **  Sets the power supply current.
        **
        ***********************************************************************
        */
        @Override
        public void setCurrent(int chan, double value)
        {
            setAmps = value;
            setPower();
        }


       /**
        ***********************************************************************
        **
        **  Sets the power supply output state.
        **
        ***********************************************************************
        */
        @Override
        public void setOutput(int chan, boolean value)
        {
            outputOn = value;
            setPower();
        }


       /**
        ***********************************************************************
        **
        **  Gets the power supply voltage.
        **
        ***********************************************************************
        */
        @Override
        public double readVoltage(int chan)
        {
            return loadVolts;
        }


       /**
        ***********************************************************************
        **
        **  Gets the power supply current.
        **
        ***********************************************************************
        */
        @Override
        public double readCurrent(int chan)
        {
            return loadAmps;
        }


       /**
        ***********************************************************************
        **
        **  Gets the power supply output state.
        **
        ***********************************************************************
        */
        @Override
        public boolean getOutput(int chan)
        {
            return outputOn;
        }


       /**
        ***********************************************************************
        **
        **  Turns the power supply on or off.
        **
        ***********************************************************************
        */
        void powerOn(boolean on)
        {
            if (!on) {
                setOutput(0, false);
            }
            setOnline(on);
        }


       /**
        ***********************************************************************
        **
        **  Sets the load power values.
        **
        ***********************************************************************
        */
        private void setPower()
        {
            if (!outputOn) {
                loadVolts = 0;
                loadAmps = 0;
            }
            else {
                double amps = setVolts / LOAD_OHMS;
                if (amps > setAmps) {
                    amps = setAmps;
                }
                loadVolts = amps * LOAD_OHMS;
                loadAmps = amps;
            }
            loadWatts = loadVolts * loadAmps;
            sim.setLoadTmpRate(loadWatts);
        }

    }


   /**
    ***************************************************************************
    **
    **  Inner container class for channel simulation data.
    **
    ***************************************************************************
    */
    class ChannelSim {

       /**
        ***********************************************************************
        **
        **  Data fields
        **
        ***********************************************************************
        */
        int      hwChan;      // HW channel number
        double   onValue;     // On (operating) value
        double   onTime;      // Time (secs) to reach on value
        double   offValue;    // Off (ambient) value
        double   offTime;     // Time (secs) to reach off value

        double   endValue;    // Ending value
        double   rate;        // Rate of change
        boolean  negative;    // True if nominal change is negative
        double   value;       // Current value
        long     currTime;    // Current system time


       /**
        ***********************************************************************
        **
        **  Constructor
        **
        ***********************************************************************
        */
        public ChannelSim(int hwChan, double onValue, double onTime,
                          double offValue, double offTime)
        {
            this.hwChan   = hwChan;
            this.onValue  = onValue;
            this.onTime   = onTime;
            this.offValue = offValue;
            this.offTime  = offTime;
            currTime      = System.currentTimeMillis();
        }


       /**
        ***********************************************************************
        **
        **  Initializes the channel.
        **
        ***********************************************************************
        */
        void initialize()
        {
            if (this == loadPwrChan) {
                onTime = 0;
                offValue = 0;
                offTime = 0;
            }
            endValue = offValue;
            value = offValue;
        }


       /**
        ***********************************************************************
        **
        **  Reads the channel.
        **
        ***********************************************************************
        */
        void read()
        {
            long thisTime = System.currentTimeMillis();
            double intvl = (thisTime - currTime) / 1000.0;
            currTime = thisTime;
            value += rate * intvl;
            if (negative) {
                if (value < endValue) {
                    value = endValue;
                }
            }
            else {
                if (value > endValue) {
                    value = endValue;
                }
            }
        }


       /**
        ***********************************************************************
        **
        **  Sets channel to power-on or power-off value.
        **
        ***********************************************************************
        */
        void powerOn()
        {
            currTime = System.currentTimeMillis();
            rate = 0;
            if (lineOn[MAIN_POWER_LINE]) {
                endValue = onValue;
                if (onTime != 0) {
                    rate = (onValue - offValue) / onTime;
                }
            }
            else {
                endValue = offValue;
                if (offTime != 0) {
                    rate = (offValue - onValue) / offTime;
                }
            }
            if (rate == 0) {
                value = endValue;
            }
            negative = rate < 0;
        }

    }


   /**
    ***************************************************************************
    **
    **  Constructor.
    **
    ***************************************************************************
    */
    public SimDevice()
    {
        fullName = "Simulated devices";
        sim = this;
    }


   /**
    ***************************************************************************
    **
    **  Performs full initialization.
    **
    ***************************************************************************
    */
    @Override
    protected void initialize()
    {
        if (loadTmpChan == null) {
            log.error("No load temperature channel defined");
        }
        if (power == null && loadPwrChan == null) {
            log.error("No load power channel defined");
        }
        setOnline(true);
        initSensors();
    }


   /**
    ***************************************************************************
    **
    **  Closes the connection.
    **
    ***************************************************************************
    */
    @Override
    protected void close()
    {
    }


   /**
    ***************************************************************************
    **
    **  Checks a channel's parameters for validity.
    **
    ***************************************************************************
    */
    @Override
    protected int[] checkChannel(String name, int hwChan, String type,
                                 String subtype) throws Exception
    {
        if (hwChan < 0 || hwChan >= N_DEV_CHANS) {
            mon.reportError(name, "hw channel number", hwChan);
        }

        String flds[] = subtype.split(":");
        try {
            double onValue = Double.valueOf(flds[0]);
            double onTime = Double.valueOf(flds[1]);
            double offValue = Double.valueOf(flds[2]);
            double offTime = Double.valueOf(flds[3]);
            ChannelSim chan = new ChannelSim(hwChan, onValue, onTime,
                                             offValue, offTime);
            chanData[hwChan] = chan;
            if (name.equals("LoadTmp")) {
                loadTmpChan = chan;
            }
            else if (name.equals("LoadPower")) {
                loadPwrChan = chan;
            }
        }
        catch (Exception e) {
            mon.reportError(name, "subtype", subtype);
        }
                
        return new int[]{0, 0};
    }


   /**
    ***************************************************************************
    **
    **  Initializes a channel.
    **
    ***************************************************************************
    */
    @Override
    protected void initChannel(int hwChan, int type, int subtype)
    {
        chanData[hwChan].initialize();
    }


   /**
    ***************************************************************************
    **
    **  Reads a channel.
    **
    ***************************************************************************
    */
    @Override
    protected double readChannel(int hwChan, int type)
    {
        ChannelSim chan = chanData[hwChan];
        chan.read();

        return chan.value;
    }


   /**
    ***************************************************************************
    **
    **  Checks an output line number.
    **
    ***************************************************************************
    */
    @Override
    protected void checkHwLine(String name, int line) throws Exception
    {
        if (line < 0 || line >= N_DEV_LINES) {
            mon.reportError(name, "line number", line);
        }
    }


   /**
    ***************************************************************************
    **
    **  Sets an output line on or off.
    **
    ***************************************************************************
    */
    @Override
    protected void setHwLine(int line, boolean on)
    {
        lineOn[line] = on;
        System.out.print("Line states:");
        for (boolean lon : lineOn) {
            System.out.print(lon ? " on" : " off");
        }
        System.out.println();
        if (line == MAIN_POWER_LINE) {
            setMainPower();
        }
        else if (line == LOAD_POWER_LINE) {
            setLoadPower();
        }
    }


   /**
    ***************************************************************************
    **
    **  Gets the set state of an output line.
    **
    ***************************************************************************
    */
    @Override
    protected boolean isHwLineSet(int line)
    {
        return lineOn[line];
    }


   /**
    ***************************************************************************
    **
    **  Sets the load power fraction
    **
    ***************************************************************************
    */
    public void setLoadFract(double fract)
    {
        loadFract = fract;
        setLoadPower();
    }


   /**
    ***************************************************************************
    **
    **  Adjusts the load temperature rate from the load power.
    **
    ***************************************************************************
    */
    void setLoadTmpRate(double loadPower)
    {
        ChannelSim tc = loadTmpChan;
        if (tc != null) {
            tc.powerOn();
            tc.rate += LOAD_POWER_FCTR * loadPower;
        }
    }

        
   /**
    ***************************************************************************
    **
    **  Turns the main power on or off.
    **
    ***************************************************************************
    */
    private void setMainPower()
    {
        for (ChannelSim chan : chanData) {
            if (chan != null && chan != loadPwrChan) {
                chan.powerOn();
            }
        }
        setLoadPower();
    }
            

   /**
    ***************************************************************************
    **
    **  Turns the load power on or off, setting the correct level.
    **
    ***************************************************************************
    */
    private void setLoadPower()
    {
        if (power != null) {
            power.powerOn(lineOn[LOAD_POWER_LINE]);
        }
        else {
            if (loadPwrChan == null) return;
            double value = 0;
            if (lineOn[MAIN_POWER_LINE]) {
                if (lineOn[LOAD_POWER_LINE]) {
                    value = loadFract * loadPwrChan.onValue;
                }
            }
            setLoadTmpRate(value);
            loadPwrChan.endValue = value;
            loadPwrChan.value = value;
        }
    }

}
